package fr.emn.spiraclock;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

/**
 *
 */

public class EMNBusIntervalPanel extends JTabbedPane {

   final EMNBusSchedule schedule;

   JSelector[] selectors = new JSelector[8];
   int selector_index = 0;
   Border defaultTextBorder = null;

   class JSelector extends JPanel {
         JTextField lbl, value;
         JButton decr, incr;
         boolean longueur;
         public JSelector(final String field, Color bg, String title, String label, boolean longueur) {
               this.longueur = longueur;
               Color bg_light = Spiral.mix(getBackground(), bg, 0.2f);
               Color bg_med = Spiral.mix(getBackground(), bg, 0.4f);
               setLayout(new BorderLayout());
               add(lbl = new JTextField(label+"   "), BorderLayout.CENTER);
               lbl.setHorizontalAlignment(JTextField.RIGHT);
               lbl.setEditable(false);
               lbl.setBackground(bg_light);
               defaultTextBorder = lbl.getBorder();
               lbl.setBorder(null);
               JPanel select = new JPanel();
               select.setLayout(new BorderLayout());
               select.add(value=new JTextField(), BorderLayout.WEST);
               select.setBackground(bg_light);
               value.setColumns(5);
               value.setHorizontalAlignment(JTextField.LEFT);
               value.setEditable(false);
               value.setFont(value.getFont().deriveFont(Font.BOLD));
               value.setBackground(bg_light);
               select.add(decr=new JButton("<") {{
                     addActionListener(new ActionListener() {
                           public void actionPerformed(ActionEvent e) {
                                 schedule.setIntervalProperty(field, -1, true);
                                 schedule.clearCache();
                                 schedule.updateView();
                           }
                     });
               }}, BorderLayout.CENTER);
               select.add(incr=new JButton(">") {{
                     addActionListener(new ActionListener() {
                           public void actionPerformed(ActionEvent e) {
                                 schedule.setIntervalProperty(field, 1, true);
                                 schedule.clearCache();
                                 schedule.updateView();
                           }
                     });
               }}, BorderLayout.EAST);
               decr.setBackground(bg_med);
               incr.setBackground(bg_med);
               add(select, BorderLayout.EAST);
               setBackground(bg_light);
               TitledBorder tb;
               setBorder(tb = BorderFactory.createTitledBorder(BorderFactory.createLoweredBevelBorder(), title));
               tb.setTitleColor(bg);
         }
   }

   class JIntervalSelector extends JPanel {
      public JIntervalSelector(String field, Color bg, String begin, String end, boolean longueur) {
            setLayout(new BorderLayout());
            add(selectors[selector_index++] = new JSelector(field + " DEBUT", bg, "Dbut", begin, false), BorderLayout.NORTH);
            add(selectors[selector_index++] = new JSelector(field + (longueur?" LONGUEUR":" FIN"), bg, longueur?"Longueur":"Fin", end, longueur), BorderLayout.SOUTH);
            this.setBackground(bg);
            this.setBorder(BorderFactory.createEmptyBorder(4,2,4,2));//.createRaisedBevelBorder());
      }
   }

   public void setField(int index, int value) {
         if (value == 0) {
               selectors[index].value.setText("");
               if (selectors[index].value.getBorder() != null)
                     selectors[index].value.setBorder(null);
         } else {
               if (selectors[index].value.getBorder() == null)
                     selectors[index].value.setBorder(defaultTextBorder);
               if (selectors[index].longueur)
                     selectors[index].value.setText("    " + value + "mn");
               else if (value > 0)
                     selectors[index].value.setText(" + " + value + "mn");
               else
                     selectors[index].value.setText(" - " + (-value) + "mn");
         }
   }

   public EMNBusIntervalPanel(EMNBusSchedule sc) {
      super();
      this.schedule = sc;

      // ADD COMPONENTS
      ClassLoader cl = getClass().getClassLoader();
      final ImageIcon i1, i2, i3;
      i1 = new ImageIcon(cl.getResource("fr/emn/spiraclock/bus0.gif"));
      i2 = new ImageIcon(cl.getResource("fr/emn/spiraclock/bus1.gif"));
      i3 = new ImageIcon(cl.getResource("fr/emn/spiraclock/bus2.gif"));

//      Color c72_76 = Spiral.transp(Spiral.mix(EMNBusSchedule.c72_regular, EMNBusSchedule.c76_regular, 0.5f), 10);
      Color c72_76 = Spiral.transp(EMNBusSchedule.c72_regular, 10);
      Color c22 = Spiral.transp(Spiral.mix(EMNBusSchedule.c22_regular, Color.black, 0.1f), 10);

      JPanel p1 = new JPanel();
      p1.setLayout(new BoxLayout(p1, BoxLayout.Y_AXIS));
      p1.add(new JIntervalSelector("72", c72_76, "72/76 Arrt Gachet", "", true));
      p1.add(new JIntervalSelector("22", c22, "72/76 Arrt G. + Attente 22", "", true));
      JPanel p2 = new JPanel();
      p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
      p2.add(new JIntervalSelector("ARRIVEES 72", c72_76, "72/76 Arrt Gachet", "72/76 Arrt Beaujoire", false));
      p2.add(new JIntervalSelector("ARRIVEES 22", c22, "22 Arrt Beaujoire", "", true));

      this.addTab("Dparts seuls", p1);
      this.addTab("Avec arrives", p2);
      setSelectedIndex(schedule.withArrivals?1:0);
      addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                  int selected = ((JTabbedPane)e.getSource()).getSelectedIndex();
                  schedule.setWithArrivals(selected == 1);
            }
      });

      // Update values
      setField(0, schedule.offset72_b);
      setField(1, schedule.offset72_l);
      setField(2, schedule.offset22_b);
      setField(3, schedule.offset22_l);
      setField(4, schedule.arr_offset72_b);
      setField(5, schedule.arr_offset72_e);
      setField(6, schedule.arr_offset22_b);
      setField(7, schedule.arr_offset22_l);
   }

  // test
  public static void main(String[] args) {
      final EMNBusSchedule schedule = new EMNBusSchedule(false);
      JDialog intervaldialog = new JDialog() {{
            setTitle("Intervalles");
            setResizable(false);
            getContentPane().add(new EMNBusIntervalPanel(schedule));
            pack();
            setVisible(true);
      }};
  }
}