package fr.emn.spiraclock;

import java.util.*;
import java.awt.Color;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.io.*;
import fr.emn.utils.file.*;

/**
 * @author Pierre Dragicevic
 * Horaires 2003-2004
 */

public class EMNBusSchedule extends Schedule implements ItemListener, ActionListener {

	public static final URL SCHEDULES = EMNBusSchedule.class.getClassLoader().getResource("fr/emn/spiraclock/horaires.txt");
	static final String DATA_INSTRUCTION = "DATA";
	static boolean data_loaded = false;
	boolean appletVersion;

   static Color c72_light = new Color(0f, 0f, 1f, 0.4f);
   static Color c72_regular = new Color(0f, 0f, 1f, 0.6f);
   static Color c76_light = new Color(0f, 0.7f, 0f, 0.6f);
   static Color c76_regular = new Color(0f, 0.7f, 0f, 0.6f);
	static Color c22_regular = new Color(1f, 0f, 0f, 0.4f);

	static final int JOUR_ROSE = 0;
	static final int JOUR_JAUNE = 1;
	static final int JOUR_VERT = 2;
	static final int JOUR_BLEU = 3;
	static final int JOUR_VIDE = 4;

	/**
	 * Mode:
	 * 0 = 72 & 76 in blue
	 * 1 = 72 in blue, 76 in green
	 * 2 = 72 & 76 in blue, 22 in red (by default)
	 * 3 = 72 in blue, 76 in green, 22 in red
	 */
	int mode = 1;
	boolean withArrivals = false;

      int offset72_b=-3, offset72_l=3, offset22_b=0, offset22_l=0;
      int arr_offset72_b=0, arr_offset72_e=0, arr_offset22_b=0, arr_offset22_l=0;

	JDialog modedialog = null;
      JDialog intervaldialog = null;
      JCheckBoxMenuItem withArrivalsItem = null;
      EMNBusModePanel modepanel = null;
      EMNBusIntervalPanel intervalpanel = null;


	/**
	 * Period tables with key dates
	 */
	static int[][] periods;

	/**
	 * Bus hours
	 */
	static int[][] bus72_gachet;
	static int[][] bus72_beaujoire;
	static int[][] bus72termBeaujoire_gachet;
	static int[][] bus72termBeaujoire_beaujoire;
	static int[][] bus76_gachet;
	static int[][] bus76_beaujoire;
	static int[][] bus22_beaujoire;
	static int[][] bus22dim_gachet;
	static int[][] bus22dim_beaujoire;

	static {load_data();}


	public EMNBusSchedule(boolean appletVersion) {
		super();
		this.appletVersion = appletVersion;
	}

	// Used in applets
	public void setModePanel(EMNBusModePanel modepanel) {
		this.modepanel = modepanel;
	}

	public static void main(String[] args) {
		if (args!=null && args.length>0 && args[0].equals("-test")) {
			testPeriods();
			return;
		}
		try{
			  Calendar c = Calendar.getInstance();
			  c.clear();
			  c.setTime(new Date(System.currentTimeMillis()));
			  String errdate = String.valueOf(c.get(Calendar.DAY_OF_YEAR))+"-"+String.valueOf(c.get(Calendar.YEAR))+"-"+String.valueOf(c.get(Calendar.HOUR_OF_DAY))+"h"+String.valueOf(c.get(Calendar.MINUTE)+"m"+String.valueOf(c.get(Calendar.SECOND)));
			  SaveOutput.start("BusClock_err.log");
			  System.out.println("BusClock v1.2_2005-SpiraClock v1.1f log file");
			  System.out.println();
			  System.out.println(c.getTime());
			  System.out.println();
			  System.out.print(System.getProperty("java.vm.name") + " version " + System.getProperty("java.vm.version") + " by " + System.getProperty("java.vm.vendor"));
			  System.out.println();
			  System.out.println();
		// Set system look & feel
		try {
			javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
		}

/*
		int mode = 2;
		boolean withArrivals = false;
		if (args.length >= 2 && args[0].equalsIgnoreCase("-mode"))
			mode = (Integer.valueOf(args[1])).intValue();
		if (args.length >= 3 && (args[2].equalsIgnoreCase("-real")))
			withArrivals = true;
*/
		SpiraWindow frame = new SpiraWindow(new EMNBusSchedule(false));
                frame.pack();
		frame.setVisible(true);
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				System.out.println();
				System.out.println(e.getClass().getName() + " message:");
				System.out.println(e.getMessage());
				System.out.println();
				System.out.println(e.getClass().getName() + " Stack Trace:");
				e.printStackTrace();
				SaveOutput.stop();
				System.exit(0);
			 }
	}

	public String getName() {
		return "Bus schedule";
	}

	/**
	 */
	public void reload() {
		load_data();
	}

      public Hashtable getProperties() {
            Hashtable result = new Hashtable();
            result.put("WITHARRIVALS", new Boolean(withArrivals));
            result.put("BUSMODE", new Integer(mode));
            result.put("72 DEBUT", new Integer(offset72_b));
            result.put("72 LONGUEUR", new Integer(offset72_l));
            result.put("22 DEBUT", new Integer(offset22_b));
            result.put("22 LONGUEUR", new Integer(offset22_l));
            result.put("ARRIVEES 72 DEBUT", new Integer(arr_offset72_b));
            result.put("ARRIVEES 72 FIN", new Integer(arr_offset72_e));
            result.put("ARRIVEES 22 DEBUT", new Integer(arr_offset22_b));
            result.put("ARRIVEES 22 LONGUEUR", new Integer(arr_offset22_l));
            return result;
      }


      /**
       */
      public void setProperties(Hashtable props) {
            if (props.get("WITHARRIVALS") != null)
                  setWithArrivals(((Boolean) props.get("WITHARRIVALS")).booleanValue());
            if (props.get("BUSMODE") != null)
                  setMode(((Integer) props.get("BUSMODE")).intValue());
            String[] interval_props = new String[] {"72 DEBUT", "72 LONGUEUR", "22 DEBUT", "22 LONGUEUR",
                  "ARRIVEES 72 DEBUT", "ARRIVEES 72 FIN", "ARRIVEES 22 DEBUT", "ARRIVEES 22 LONGUEUR"};
            for(int i=0; i<interval_props.length; i++)
                  if (props.get(interval_props[i]) != null)
                        setIntervalProperty(interval_props[i], ((Integer) props.get(interval_props[i])).intValue(), false);
            clearCache();
		updateView();
	}

      public void setIntervalProperty(String field, int i, boolean add) {
//            int offset72_b, offset72_l, offset22_b, offset22_l;
//            int arr_offset72_b, arr_offset72_e, arr_offset22_b, arr_offset22_l;
            if (field.equals("72 DEBUT")) {
                  if (add) offset72_b += i; else offset72_b = i;
                  if (intervalpanel != null) intervalpanel.setField(0, offset72_b);
            }
            if (field.equals("72 LONGUEUR")) {
                  if (add) offset72_l = Math.max(0, offset72_l+i); else offset72_l = i;
                  if (intervalpanel != null) intervalpanel.setField(1, offset72_l);
            }
            if (field.equals("22 DEBUT")) {
                  if (add) offset22_b += i; else offset22_b = i;
                  if (intervalpanel != null) intervalpanel.setField(2, offset22_b);
            }
            if (field.equals("22 LONGUEUR")) {
                  if (add) offset22_l = Math.max(0, offset22_l+i); else offset22_l = i;
                  if (intervalpanel != null) intervalpanel.setField(3, offset22_l);
            }
            if (field.equals("ARRIVEES 72 DEBUT")) {
                  if (add) arr_offset72_b += i; else arr_offset72_b = i;
                  if (intervalpanel != null) intervalpanel.setField(4, arr_offset72_b);
            }
            if (field.equals("ARRIVEES 72 FIN")) {
                  if (add) arr_offset72_e += i; else arr_offset72_e = i;
                  if (intervalpanel != null) intervalpanel.setField(5, arr_offset72_e);
            }
            if (field.equals("ARRIVEES 22 DEBUT")) {
                  if (add) arr_offset22_b += i; else arr_offset22_b = i;
                  if (intervalpanel != null) intervalpanel.setField(6, arr_offset22_b);
            }
            if (field.equals("ARRIVEES 22 LONGUEUR")) {
                  if (add) arr_offset22_l = Math.max(0, arr_offset22_l+i); else arr_offset22_l = i;
                  if (intervalpanel != null) intervalpanel.setField(7, arr_offset22_l);
            }
      }


      public void beforeClosing(){
	  SaveOutput.stop();
	}

	/**
	 *
	 */
	public void installPopupCommands(final JPopupMenu popup) {
		popup.add(withArrivalsItem = new JCheckBoxMenuItem("Avec arrives", this.withArrivals) {{
			addItemListener(EMNBusSchedule.this);
		}});
		if (!appletVersion)
			popup.add(new JMenuItem("Lignes...") {{
				addActionListener(EMNBusSchedule.this);
			}});
      popup.add(new JMenuItem("Intervalles...") {{
                  addActionListener(EMNBusSchedule.this);
		}});
	}

     public void setWithArrivals(boolean v) {
            if (withArrivals != v) {
                  withArrivals = v;
                  clearCache();
                  updateView();
            }
            if (withArrivalsItem != null && withArrivalsItem.getState() != v)
                  withArrivalsItem.setState(v);
            if (intervalpanel != null && v != (intervalpanel.getSelectedIndex()==1) )
                  intervalpanel.setSelectedIndex(v?1:0);
      }

      public void setMode(int v) {
            if (mode != v) {
                  mode = v;
                  clearCache();
                  updateView();
            }
            if (modepanel != null)
                  modepanel.select(v);
      }


	/**
	 *
	 */
	public Appointment[] getDayAppointments(ATime day) {
		if (!data_loaded)
			return new Appointment[]{};
		int period = getPeriod(day);
//System.out.println(day.getDayOfWeek() + " PERIOD: " + period);
		if (period == JOUR_VIDE)
			return new Appointment[]{};
		Color c72;
		if (withArrivals)
			c72 = c72_light;
		else
			c72 = c72_regular;
		Color c76;
		if (mode == 1 || mode == 3) {
			if (withArrivals)
				c76 = c76_light;
			else
				c76 = c76_regular;
		} else
			c76 = c72;
		Color c22 = c22_regular;
		Vector apps = new Vector();
		int year = day.year;
		int dayofyear = day.dayOfYear;
		int start, end;
		String sum, sum2;

		for (int i=0; i<bus72_gachet[period].length; i++) {
			start = bus72_gachet[period][i];
			end = bus72_beaujoire[period][i];
			sum = "Bus 72 - Gachet " + str(start);
			sum2 = sum + " / Beaujoire " + str(end);
			if (withArrivals)
				apps.add(getNew(sum2, year, dayofyear, start +arr_offset72_b, end +arr_offset72_e, c72, false, true, false, false));
			else
				apps.add(getNew(sum, year, dayofyear, start +offset72_b, start +offset72_l+offset72_b, c72,false, false, false, true));
		}
		for (int i=0; i<bus72termBeaujoire_gachet[period].length; i++) {
			start = bus72termBeaujoire_gachet[period][i];
			end = bus72termBeaujoire_beaujoire[period][i];
			sum = "Bus 72 - Gachet " + str(start) + " / Terminus Beaujoire";
			sum2 = sum + " " + str(end);
			if (withArrivals)
				apps.add(getNew(sum2, year, dayofyear, start +arr_offset72_b, end +arr_offset72_e, c76, false, true, false, false));
			else
				apps.add(getNew(sum, year, dayofyear, start +offset72_b, start +offset72_l+offset72_b, c76, false, false, false, true));
		}
		for (int i=0; i<bus76_gachet[period].length; i++) {
			start = bus76_gachet[period][i];
			end = bus76_beaujoire[period][i];
			sum = "Bus 76 - Gachet " + str(start);
			sum2 = sum + " / Beaujoire " + str(end);
			if (withArrivals)
				apps.add(getNew(sum2, year, dayofyear, start +arr_offset72_b, end +arr_offset72_e, c76, false, true, false, false));
			else
				apps.add(getNew(sum, year, dayofyear, start +offset72_b, start +offset72_l+offset72_b, c76, false, false, false, true));
		}
		for (int i=0; i<bus22dim_gachet[period].length; i++) {
			start = bus22dim_gachet[period][i];
			end = bus22dim_beaujoire[period][i];
			sum = "Bus 22 - Gachet " + str(start);
			sum2 = sum + " / Beaujoire " + str(end);
			if (withArrivals)
				apps.add(getNew(sum2, year, dayofyear, start +arr_offset72_b, end +arr_offset72_e, c76, false, true, false, false));
			else
				apps.add(getNew(sum, year, dayofyear, start +offset72_b, start +offset72_l+offset72_b, c76, false, false, false, true));
		}
		if (mode == 2 || mode == 3) {
			int timeto22 = 0;
			for (int i=0; i<bus22_beaujoire[period].length; i++) {
				start = bus22_beaujoire[period][i];
				sum = "Bus 22 - Beaujoire " + str(start);
				if (withArrivals)
					apps.add(getNew(sum, year, dayofyear, start +arr_offset22_b, start +arr_offset22_b+arr_offset22_l, c22, true, false, false, false));
				else {
					timeto22 = getTimeTo22(period, i);
					apps.add(getNew(sum, year, dayofyear, start-timeto22 +offset22_b, start-timeto22  +offset22_b+offset22_l, c22, true, false, false, true));
				}
			}
		}
		//
		Appointment[] array = new Appointment[apps.size()];
		int i=0;
		for (Iterator it = apps.iterator(); it.hasNext(); i++) {
			array[i] = (Appointment)it.next();
		}
		return array;
	}

	/**
	 *
	 */
	protected Appointment getNew(String summary, int year, int dayofyear, int mmstart, int mmend, Color c, boolean overlap, boolean showStart, boolean showEnd, boolean fadeOnEnd) {		//int mmstart = bushour+start;
		//int mmend = bushour+end;
		int sstart = 0;
		int send = 0;
		if (mmend <= mmstart) {
			mmend = mmstart;
			send = 30;
			mmstart--;
			sstart = 30;
		}
		ATime startt = new ATime(year, dayofyear, (mmstart)/60, (mmstart)%60, sstart);
		ATime endt = new ATime(year, dayofyear, (mmend)/60, (mmend)%60, send);
		Appointment a = new Appointment(startt, endt, summary, "", c, overlap, showStart, showEnd, fadeOnEnd);
//		if (pos)
//			a.pos = priority*1.1/100.0;
		return a;
	}

	protected static int getPeriod(ATime day) {
		ATime d = new ATime(day);
		d.hours = 0;
		d.minutes = 0;
		d.seconds = 0;
		//
//System.out.println(day);
		int p = JOUR_VIDE;
		if (!isMatchingTable(d, JOUR_ROSE) || isMatchingTable(d, JOUR_VIDE))
			p = JOUR_VIDE;
		else {
			p = JOUR_ROSE;
			if (isMatchingTable(d, JOUR_JAUNE))
				p = JOUR_JAUNE;
			if (d.getDayOfWeek()==Calendar.SATURDAY || isMatchingTable(d, JOUR_VERT))
				p = JOUR_VERT;
			if (d.getDayOfWeek()==Calendar.SUNDAY || isMatchingTable(d, JOUR_BLEU))
				p = JOUR_BLEU;
		}
		return p;
	}

	static void testPeriods() {
		String[] names = new String[] {"R", "Jaune", "Vert", "Bleu", "-"};
		ATime start = (new ATime(2000+periods[0][2], periods[0][1], periods[0][0])).add(-1);
		ATime end = (new ATime(2000+periods[0][5], periods[0][4], periods[0][3])).add(2);
		ATime t = start;
		int month = -1;
		while (!t.equals(end)) {
			System.out.println(t.getDateString() + " " + names[getPeriod(t)]);
			t = t.add(1);
			if (month != t.getMonth()) {
				if (month != -1) {
					System.out.println();
					System.out.println("<Appuyez sur entre>");
					try{
						while(System.in.available() > 0)
							System.in.read();
						System.in.read();
					} catch (Exception e) {
					}
					System.out.println();
				}
				month = t.getMonth();
			}
		}
	}

	private static boolean isMatchingTable(ATime day, int period) {
		if (!data_loaded)
			return false;
		int[] data = periods[period];
		ATime start;
		ATime end;
		for (int i=0; i<data.length; i+=6) {
			start = new ATime(2000+data[i+2], data[i+1], data[i+0]);
			end = new ATime(2000+data[i+5], data[i+4], data[i+3]);
			if (day.isBetween(start, end))
				return true;
		}
		return false;
	}

	private static int getTimeTo22(int period, int index) {
		int bushour = bus22_beaujoire[period][index];
		int[] h72 = getNearestBus(bus72_beaujoire[period], bushour);
		int[] h76 = getNearestBus(bus76_beaujoire[period], bushour);
		int[] h22dim = getNearestBus(bus22dim_beaujoire[period], bushour);
		if (h72[1]<1000 && h72[1]<=h76[1] && h72[1]<=h22dim[1])
			return bus72_beaujoire[period][h72[0]] - bus72_gachet[period][h72[0]];
		if (h76[1]<1000 && h76[1]<=h72[1] && h76[1]<=h22dim[1])
			return bus76_beaujoire[period][h76[0]] - bus76_gachet[period][h76[0]];
		if (h22dim[1]<1000 && h22dim[1]<=h72[1] && h22dim[1]<=h76[1])
			return bus22dim_beaujoire[period][h22dim[0]] - bus22dim_gachet[period][h22dim[0]];
		return 10;
	}

	// bus index + hour distance in mn
	private static int[] getNearestBus(int[] bushours, int hour) {
		int i;
		for (i=0; i<bushours.length && bushours[i]<=hour; i++);
		i--;
		if (i > -1)
			return new int[] {i, (int)(Math.abs(bushours[i] - hour))};
		return new int[] {-1, 1000};
	}

	// POPUPMENU LISTENERS

	/**
	 * Checkboxes
	 */
	public void itemStateChanged(ItemEvent e) {
		String command = ((JMenuItem)e.getSource()).getText();
		if (command.equals("Avec arrives")) {
			setWithArrivals(e.getStateChange() == e.SELECTED);
		}
	}

	public void actionPerformed(final ActionEvent e) {
		String command = ((JMenuItem)e.getSource()).getText();
		if (command.equals("Lignes...")) {
			if (modepanel == null) {
				modedialog = new JDialog() {{
					setTitle("Lignes");
					setResizable(false);
					getContentPane().add(modepanel = new EMNBusModePanel(EMNBusSchedule.this, false));
					pack();
					setVisible(true);
				}};
			}
			if (modedialog != null) {
				modedialog.setVisible(true);
         	modedialog.setLocation(getView().getBestSideLocation(modedialog, null));
         }
		}
            if (command.equals("Intervalles...")) {
                  if (intervaldialog == null) {
                        intervaldialog = new JDialog() {{
                              setTitle("Intervalles");
                              setResizable(false);
                              getContentPane().add(intervalpanel = new EMNBusIntervalPanel(EMNBusSchedule.this));
                              pack();
                              setVisible(true);
                        }};
                  }
                  intervaldialog.setVisible(true);
                  intervaldialog.setLocation(getView().getBestSideLocation(intervaldialog, null));
		}
	}


	//////////////////////////////////////////////////////////////////////////////////////////





	protected static void load_data() {
		data_loaded = false;
		BufferedReader breader = null;
		try {
			for (int i=0; i<10 && breader == null; i++) {
		   	breader = new BufferedReader(new InputStreamReader(SCHEDULES.openStream()));
		   	Thread.currentThread().sleep(10);
//System.err.println(breader);
		   }

  		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Impossible de trouver le fichier d'horaires");
			errorMessage = "Impossible de trouver le fichier d'horaires";
			return;
		}
//		JOptionPane.showMessageDialog(null, "Erreur fatale: votre navigateur n'est pas compatible SpiraClock");
//		JOptionPane.showConfirmDialog(null, "Voulez-vous recompiler le noyau ?");


		try {
			// Read calendar periods
			periods = new int[5][];
			periods[JOUR_ROSE] = readPeriod(breader);
			periods[JOUR_JAUNE] = readPeriod(breader);
			periods[JOUR_VERT] = readPeriod(breader);
			periods[JOUR_BLEU] = readPeriod(breader);
			periods[JOUR_VIDE] = readPeriod(breader);

			// Read bus schedules
			bus72_gachet = readSchedule(breader);
			bus72_beaujoire = readSchedule(breader);
			bus72termBeaujoire_gachet = readSchedule(breader);
			bus72termBeaujoire_beaujoire = readSchedule(breader);
			bus76_gachet = readSchedule(breader);
			bus76_beaujoire = readSchedule(breader);
			bus22_beaujoire = readSchedule(breader);
			bus22dim_gachet = readSchedule(breader);
			bus22dim_beaujoire = readSchedule(breader);
			breader.close();
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Erreur lors du chargement d'horaires");
			errorMessage = "Erreur lors du chargement d'horaires";
			return;
		}
		data_loaded = true;
	}

	private static int[] readPeriod(BufferedReader breader)  throws java.io.IOException {
		return parseDates(nextData(breader, "["));
	}

	private static int[][] readSchedule(BufferedReader breader)  throws java.io.IOException {
		int[][] res = new int[4][];
		res[JOUR_ROSE] = parseHours(nextData(breader, " "));
		res[JOUR_VERT] = parseHours(nextData(breader, " "));
		res[JOUR_JAUNE] = parseHours(nextData(breader, " "));
		res[JOUR_BLEU] = parseHours(nextData(breader, " "));
		return res;
//		return new int[][] {parseHours(nextData(breader, " ")), parseHours(nextData(breader, " ")), parseHours(nextData(breader, " ")), parseHours(nextData(breader, " "))};
	}

	// Returns next DATA line
	private static String nextDataLine(BufferedReader breader) throws java.io.IOException {
		String line;
	  	while (breader.ready() && (line=breader.readLine()) != null) {
			if (line.trim().startsWith(DATA_INSTRUCTION)) {
				int begin = line.indexOf(DATA_INSTRUCTION)+DATA_INSTRUCTION.length()+1;
				if (begin >= line.length())
					return "";
				return line.substring(begin).trim();
			}
		}
		return null;
	}

	// Tokenizes and returns next DATA line
	private static String[] nextData(BufferedReader breader, String delimiter) throws java.io.IOException {
		String line = nextDataLine(breader);
		StringTokenizer tokens = new StringTokenizer(line.trim(), delimiter);
		int n = tokens.countTokens();
		String[] data = new String[n];
		for (int i=0; i<n; i++)
			data[i] = tokens.nextToken();
		return data;
	}

	private static int[] parseDates(String[] tokens) {
		String token;
		Vector ints = new Vector();
		Vector tmp_ints = new Vector();

		// Parse date [...] tokens
		for (int i=0; i<tokens.length; i++) {
			token = tokens[i].trim();
			// Remove [ and ] separators
			if (token.startsWith("["))
				token = token.substring(1);
			if (token.endsWith("]"))
				token = token.substring(0, token.length()-1);
			// Convert tokens inside [ ] into a vector of Integers
			tmp_ints.clear();
			StringTokenizer tokens_inside = new StringTokenizer(token, " ");
			int n = tokens_inside.countTokens();
			for (int j=0; j<n; j++) {
				try { // Ignore non-numeric tokens
					int num = Integer.parseInt(tokens_inside.nextToken());
					tmp_ints.add(new Integer(num));
				} catch (NumberFormatException e) {
				}
			}
			// If 6 integers (date interval), add them all
			if (tmp_ints.size() == 6)
				ints.addAll(tmp_ints);
			// If 3 integers (single date), add them twice
			else if (tmp_ints.size() == 3) {
				ints.addAll(tmp_ints);
				ints.addAll(tmp_ints);
			}
			else return null; // this should not happen
		}
		// Convert the Integer vector into an array
		int[] ret = new int[ints.size()];
		for (int i=0; i<ret.length; i++)
			ret[i] = ((Integer)ints.elementAt(i)).intValue();
		return ret;
   }

	private static int[] parseHours(String[] tokens) {
		int[] res = new int[tokens.length];
		for (int i=0; i<tokens.length; i++)
			res[i] = toMinutes(tokens[i]);
		return res;
	}

	// Converts a String in HMM or HHMM format into minutes format (HH*60+MM).
	private static int toMinutes(String hhmm) {
		hhmm = hhmm.trim();
		int l = hhmm.length();
		int hours, mins;
		try {
			hours = Integer.parseInt(hhmm.substring(0, l-2));
		} catch (Exception e) {
			e.printStackTrace();
			hours = 0;
		}
		try {
			mins = Integer.parseInt(hhmm.substring(l-2));
		} catch (Exception e) {
			e.printStackTrace();
			mins = 0;
		}
		return hours*60 + mins;
	}


	///////////////////////////////////////////////////////////////////////////////////////////

	/**
	 * Converts a Minutes format into a readable String format
	 */
	private static String str(int minutes) {
		return twodigit(minutes/60)+ ":" + twodigit(minutes%60);
	}

	private static String twodigit(int n) {
		if (n<10)
			return "0" + String.valueOf(n);
		else
			return String.valueOf(n);
	}
}
